/********************************************************************************
** Form generated from reading UI file 'layer_info.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LAYER_INFO_H
#define UI_LAYER_INFO_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTableWidget>

QT_BEGIN_NAMESPACE

class Ui_LayerInfoDialog
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout1;
    QHBoxLayout *hboxLayout;
    QLabel *nameLabel;
    QLineEdit *layerNameEdit;
    QHBoxLayout *hboxLayout1;
    QLabel *stylesLabel;
    QComboBox *comboBox;
    QTableWidget *tableWidget;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *LayerInfoDialog)
    {
        if (LayerInfoDialog->objectName().isEmpty())
            LayerInfoDialog->setObjectName("LayerInfoDialog");
        LayerInfoDialog->resize(446, 344);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(LayerInfoDialog->sizePolicy().hasHeightForWidth());
        LayerInfoDialog->setSizePolicy(sizePolicy);
        LayerInfoDialog->setSizeGripEnabled(false);
        gridLayout = new QGridLayout(LayerInfoDialog);
        gridLayout->setObjectName("gridLayout");
        gridLayout1 = new QGridLayout();
        gridLayout1->setObjectName("gridLayout1");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        nameLabel = new QLabel(LayerInfoDialog);
        nameLabel->setObjectName("nameLabel");

        hboxLayout->addWidget(nameLabel);

        layerNameEdit = new QLineEdit(LayerInfoDialog);
        layerNameEdit->setObjectName("layerNameEdit");

        hboxLayout->addWidget(layerNameEdit);


        gridLayout1->addLayout(hboxLayout, 0, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        stylesLabel = new QLabel(LayerInfoDialog);
        stylesLabel->setObjectName("stylesLabel");

        hboxLayout1->addWidget(stylesLabel);

        comboBox = new QComboBox(LayerInfoDialog);
        comboBox->setObjectName("comboBox");

        hboxLayout1->addWidget(comboBox);


        gridLayout1->addLayout(hboxLayout1, 1, 0, 1, 1);

        tableWidget = new QTableWidget(LayerInfoDialog);
        tableWidget->setObjectName("tableWidget");

        gridLayout1->addWidget(tableWidget, 2, 0, 1, 1);

        buttonBox = new QDialogButtonBox(LayerInfoDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout1->addWidget(buttonBox, 3, 0, 1, 1);


        gridLayout->addLayout(gridLayout1, 0, 0, 1, 1);


        retranslateUi(LayerInfoDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, LayerInfoDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, LayerInfoDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(LayerInfoDialog);
    } // setupUi

    void retranslateUi(QDialog *LayerInfoDialog)
    {
        LayerInfoDialog->setWindowTitle(QCoreApplication::translate("LayerInfoDialog", "Layer Info", nullptr));
        nameLabel->setText(QCoreApplication::translate("LayerInfoDialog", "Name:", nullptr));
        stylesLabel->setText(QCoreApplication::translate("LayerInfoDialog", "Styles:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LayerInfoDialog: public Ui_LayerInfoDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LAYER_INFO_H
